/*
 * Copyright (c) 1995, 1996 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <assert.h>
#include <string.h>

#include <mom/libmint.h>
#include <mom/c/libcast.h>
#include <mom/c/pbe.hh>

void mu_state::mu_var_discriminate(mint_const *discrim_vals, cast_type discrim_ctype,
				   cast_expr discrim_expr, functor **success,
				   functor *failure, int count) 
{
	mu_memory_allocator_state *enter_case_state;
	cast_stmt saved_c_block, switch_body_c_block, case_c_block;
	
	// This needs to deal with non-atomic mint consts's SOON!!!  XXX
	// But for right now, here's the deal:
	assert (discrim_ctype->kind == CAST_TYPE_PRIMITIVE);

	this->break_chunk();
	enter_case_state = this->memory_allocator_state();
	saved_c_block = this->c_block;
	this->c_block = cast_new_block(0,0);
	for (int i = 0; i < count; i++) {
		// Save the body of the switch statement
		switch_body_c_block = this->c_block;
		
		// Start a new block for this case
		this->c_block = cast_new_block(0, 0);
		
		// Reset the memory state for this case
		this->set_memory_allocator_state(enter_case_state);
		
		// Create a CAST version of the mint constant
		cast_expr const_val = make_case_value_expr(discrim_vals[i]);
		
		// Call the functor to build the code to deal with this branch
		this->mu_union_case(success[i], discrim_vals[i]);
		
		// Append a 'break' and prepend the 'case' statement
		this->add_stmt(cast_new_break());
		case_c_block = this->c_block;
		this->c_block = switch_body_c_block;
		this->add_stmt(cast_new_case(const_val, case_c_block));
	}
	
	// Now deal with the default case
	// Save the body of the switch
	switch_body_c_block = this->c_block;
	// Start a new block...
	this->c_block = cast_new_block(0, 0);
	// Reset memory state
	this->set_memory_allocator_state(enter_case_state);
	// Call the functor for the default.  Note that we have to 'borrow' a constant
	this->mu_union_case(failure, discrim_vals[0]);
	// Append a 'break' and prepend the 'default' statement
	this->add_stmt(cast_new_break());
	case_c_block = this->c_block;
	this->c_block = switch_body_c_block;
	this->add_stmt(cast_new_default(case_c_block));
	
	// Produce the complete 'switch' statement
	switch_body_c_block = this->c_block;
	this->c_block = saved_c_block;
	this->add_stmt(cast_new_switch(discrim_expr, switch_body_c_block));
	
	// Cleanup
	delete enter_case_state;
}
