/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 */

#ifndef __flick_pres_corba_h
#define __flick_pres_corba_h

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <flick/pres/all.h>

/* This is a pseudo-CORBA allocator -- It doesn't implement freeing of ports */

#define CORBA_flick_alloc(a) (void *)malloc(a)
#define CORBA_flick_free(a) free(a)

#define CORBA_free(a) free(a)
#define CORBA_alloc(a) malloc(a)

/*
 * Here are the CORBA basic type specifiers.  See Section 14.7 of the CORBA 2.0
 * specification.
 */
typedef short CORBA_short;
typedef int CORBA_long;
typedef unsigned short CORBA_unsigned_short;
typedef unsigned int CORBA_unsigned_long;
typedef float CORBA_float;
typedef double CORBA_double;
typedef char CORBA_char;
typedef char CORBA_boolean;
typedef unsigned char CORBA_octet;
typedef unsigned int CORBA_enum;
/*
 * typedef struct CORBA_any {
 *         CORBA_TypeCode_type _type;
 *         void *_value;
 * } CORBA_any;
 */
#ifndef _typedef___CORBA_Object
#define _typedef___CORBA_Object
typedef FLICK_CLIENT CORBA_Object;
#endif

/*
 * Here's the Object interface that applies (non-DII/DSI/IR stuff...)
 */
CORBA_boolean CORBA_Object_is_nil(CORBA_Object ths);		/* Done */
CORBA_Object CORBA_Object_duplicate(CORBA_Object ths);		/* Done */
void CORBA_Object_release(CORBA_Object ths);			/* Done */
CORBA_unsigned_long CORBA_Object_hash(CORBA_Object ths,
				      CORBA_unsigned_long maximum);  /* Done */
CORBA_boolean CORBA_Object_is_equivalent(CORBA_Object ths,
					 CORBA_Object other_object); /* Done */
/*
 * I'm not doing this stuff right now:
 *
 * CORBA_boolean CORBA_Object_is_a(CORBA_Object ths, char *logical_type_id);
 * CORBA_boolean CORBA_Object_non_existent(CORBA_Object ths);
 */

/* Here's the stuff for exceptions */

typedef CORBA_unsigned_long CORBA_completion_status;
#define CORBA_COMPLETED_YES ((CORBA_completion_status)0)
#define CORBA_COMPLETED_NO ((CORBA_completion_status)1)
#define CORBA_COMPLETED_MAYBE ((CORBA_completion_status)2)

typedef CORBA_unsigned_long CORBA_exception_type;
#define CORBA_NO_EXCEPTION ((CORBA_exception_type)0)
#define CORBA_USER_EXCEPTION ((CORBA_exception_type)1)
#define CORBA_SYSTEM_EXCEPTION ((CORBA_exception_type)2)

typedef struct CORBA_system_exception {
	CORBA_unsigned_long minor;
	CORBA_completion_status completed;
} CORBA_system_exception;

#define sys_except(name, nm)		\
typedef CORBA_system_exception name;

sys_except(CORBA_UNKNOWN, "CORBA_UNKNOWN");
#define ex_CORBA_UNKNOWN 1
sys_except(CORBA_BAD_PARAM, "CORBA_BAD_PARAM");
#define ex_CORBA_BAD_PARAM 2
sys_except(CORBA_NO_MEMORY, "CORBA_NO_MEMORY");
#define ex_CORBA_NO_MEMORY 3
sys_except(CORBA_IMP_LIMIT, "CORBA_IMP_LIMIT");
#define ex_CORBA_IMP_LIMIT 4
sys_except(CORBA_COMM_FAILURE, "CORBA_COMM_FAILURE");
#define ex_CORBA_COMM_FAILURE 5
sys_except(CORBA_INV_OBJREF, "CORBA_INV_OBJREF");
#define ex_CORBA_INV_OBJREF 6
sys_except(CORBA_NO_PERMISSION, "CORBA_NO_PERMISSION");
#define ex_CORBA_NO_PERMISSION 7
sys_except(CORBA_INTERNAL, "CORBA_INTERNAL");
#define ex_CORBA_INTERNAL 8
sys_except(CORBA_MARSHAL, "CORBA_MARSHAL");
#define ex_CORBA_MARSHAL 9
sys_except(CORBA_INITIALIZE, "CORBA_INITIALIZE");
#define ex_CORBA_INITIALIZE 10
sys_except(CORBA_NO_IMPLEMENT, "CORBA_NO_IMPLEMENT");
#define ex_CORBA_NO_IMPLEMENT 11
sys_except(CORBA_BAD_TYPECODE, "CORBA_BAD_TYPECODE");
#define ex_CORBA_BAD_TYPECODE 12
sys_except(CORBA_BAD_OPERATION, "CORBA_BAD_OPERATION");
#define ex_CORBA_BAD_OPERATION 13
sys_except(CORBA_NO_RESOURCES, "CORBA_NO_RESOURCES");
#define ex_CORBA_NO_RESOURCES 14
sys_except(CORBA_NO_RESPONSE, "CORBA_NO_RESPONSE");
#define ex_CORBA_NO_RESPONSE 15
sys_except(CORBA_PERSIST_STORE, "CORBA_PERSIST_STORE");
#define ex_CORBA_PERSIST_STORE 16
sys_except(CORBA_BAD_INV_ORDER, "CORBA_BAD_INV_ORDER");
#define ex_CORBA_BAD_INV_ORDER 17
sys_except(CORBA_TRANSIENT, "CORBA_TRANSIENT");
#define ex_CORBA_TRANSIENT 18
sys_except(CORBA_FREE_MEM, "CORBA_FREE_MEM");
#define ex_CORBA_FREE_MEM 19
sys_except(CORBA_INV_IDENT, "CORBA_INV_IDENT");
#define ex_CORBA_INV_IDENT 20
sys_except(CORBA_INV_FLAG, "CORBA_INV_FLAG");
#define ex_CORBA_INV_FLAG 21
sys_except(CORBA_INTF_REPOS, "CORBA_INTF_REPOS");
#define ex_CORBA_INTF_REPOS 22
sys_except(CORBA_BAD_CONTEXT, "CORBA_BAD_CONTEXT");
#define ex_CORBA_BAD_CONTEXT 23
sys_except(CORBA_OBJ_ADAPTER, "CORBA_OBJ_ADAPTER");
#define ex_CORBA_OBJ_ADAPTER 24
sys_except(CORBA_DATA_CONVERSION, "CORBA_DATA_CONVERSION");
#define ex_CORBA_DATA_CONVERSION 25
sys_except(CORBA_OBJECT_NOT_EXIST, "CORBA_OBJECT_NOT_EXIST");
#define ex_CORBA_OBJECT_NOT_EXIST 26

#define FLICK_ERROR_CONSTANT		1
#define FLICK_ERROR_VIRTUAL_UNION	2
#define FLICK_ERROR_STRUCT_UNION	3
#define FLICK_ERROR_DECODE_SWITCH	4
#define FLICK_ERROR_COLLAPSED_UNION	5
#define FLICK_ERROR_VOID_UNION		6
#define FLICK_ERROR_COMMUNICATION	7

/* This is the environment variable */
typedef struct CORBA_Environment {
	CORBA_exception_type _major;
	void *_except;
	char *_user_id; /* NOTE THAT THIS IS NOT USED YET!!! */
	int _atomic_user_id; /* This is what is currently used... */
} CORBA_Environment;

#define flick_corba_init_environment() {	\
	_ev->_major = CORBA_NO_EXCEPTION;	\
	_ev->_except = 0;			\
	_ev->_user_id = 0;			\
	_ev->_atomic_user_id = 0;		\
}

#define CORBA_exception_value(_ev) (_ev)->_except
/* This one isn't right - it's supposed to use a string... */
#define CORBA_exception_id(_ev) (_ev)->_atomic_user_id
/* Frees memory taken by a potential exception */
#define CORBA_exception_free(_ev) {		\
	if ((_ev)->_except) {			\
		free((_ev)->_except);		\
		(_ev)->_except = 0;		\
	}					\
}

/* Here are the exception errors */

/* The encode/decode system exceptions don't have to deal with the _major field
 * because it has already been marshaled/unmarshaled.
 */
#define flick_corba_decode_system_exception(loc, ENCNAME, LINKNAME)	      \
{									      \
	(loc)->_except = (void *)					      \
			 CORBA_flick_alloc(sizeof(CORBA_system_exception));   \
									      \
	flick_##LINKNAME##_decode_new_glob(12);				      \
	flick_##LINKNAME##_decode_new_chunk(12);			      \
	flick_##ENCNAME##_decode_unsigned32				      \
		(0, (loc)->_atomic_user_id);				      \
	flick_##ENCNAME##_decode_unsigned32				      \
		(4, ((CORBA_system_exception *)((loc)->_except))->minor);     \
	flick_##ENCNAME##_decode_unsigned32				      \
		(8, ((CORBA_system_exception *)((loc)->_except))->completed); \
	flick_##LINKNAME##_decode_end_chunk(12);			      \
	flick_##LINKNAME##_decode_end_glob(12);				      \
}

#define flick_corba_encode_system_exception(loc, ENCNAME, LINKNAME)	      \
{									      \
	flick_##LINKNAME##_encode_new_glob(12);				      \
	flick_##LINKNAME##_encode_new_chunk(12);			      \
	flick_##ENCNAME##_encode_unsigned32 				      \
		(0, (loc)->_atomic_user_id);				      \
	flick_##ENCNAME##_encode_unsigned32				      \
		(4, ((CORBA_system_exception *)((loc)->_except))->minor);     \
	flick_##ENCNAME##_encode_unsigned32				      \
		(8, ((CORBA_system_exception *)((loc)->_except))->completed); \
	flick_##LINKNAME##_encode_end_chunk(12);			      \
	flick_##LINKNAME##_encode_end_glob(12);				      \
									      \
	CORBA_flick_free(loc);						      \
}


/* These are all the error macros */

#define flick_corba_server_error(retname, errval, complete, \
				 ENCNAME, LINKNAME) {			 \
	flick_##LINKNAME##_server_start_encode();			 \
	flick_##LINKNAME##_encode_new_glob_plain(16);			 \
	flick_##LINKNAME##_encode_new_chunk_plain(16);			 \
	flick_##ENCNAME##_encode_unsigned32(0, CORBA_SYSTEM_EXCEPTION);	 \
	/* XXX --- We should pass in the system exception type! */	 \
	flick_##ENCNAME##_encode_unsigned32(4, ex_CORBA_MARSHAL);	 \
	flick_##ENCNAME##_encode_unsigned32(8, errval); /* minor code */ \
	flick_##ENCNAME##_encode_unsigned32(12, complete);		 \
	flick_##LINKNAME##_encode_end_chunk_plain(16);			 \
	flick_##LINKNAME##_encode_end_glob_plain(16);			 \
	flick_##LINKNAME##_server_end_encode();				 \
	retname;							 \
}

#define flick_corba_encode_server_error(retname, errval, ENCNAME, LINKNAME)\
	flick_corba_server_error(retname, errval, CORBA_COMPLETED_YES,	\
				 ##ENCNAME, ##LINKNAME)

#define flick_corba_decode_server_error(retname, errval, ENCNAME, LINKNAME)	\
	flick_corba_server_error(retname, errval, CORBA_COMPLETED_NO,		\
				 ##ENCNAME, ##LINKNAME)

#define flick_corba_other_server_error(retname, errval, ENCNAME, LINKNAME)	\
	flick_corba_server_error(retname, errval, CORBA_COMPLETED_MAYBE,	\
				 ##ENCNAME, ##LINKNAME)


#define flick_corba_client_error(retname, errval, complete, ENCNAME, LINKNAME) {	\
	_ev->_major = CORBA_SYSTEM_EXCEPTION;						\
	_ev->_atomic_user_id = ex_CORBA_MARSHAL; /* XXX --- Should pass this in. */	\
	_ev->_except = (void*)CORBA_flick_alloc(sizeof(CORBA_system_exception));	\
	((CORBA_system_exception *)_ev->_except)->minor = (errval);			\
	((CORBA_system_exception *)_ev->_except)->completed = (complete);			\
	retname;									\
}

#define	flick_corba_encode_client_error(retname, errval, ENCNAME, LINKNAME)	\
	flick_corba_client_error(retname, errval, CORBA_COMPLETED_NO,		\
				 ##ENCNAME, ##LINKNAME)


#define flick_corba_decode_client_error(retname, errval, ENCNAME, LINKNAME)	\
	flick_corba_client_error(retname, errval, CORBA_COMPLETED_MAYBE,	\
				 ##ENCNAME, ##LINKNAME)


#define flick_corba_other_client_error(retname, errval, ENCNAME, LINKNAME)	\
	flick_corba_client_error(retname, errval, CORBA_COMPLETED_MAYBE,	\
				 ##ENCNAME, ##LINKNAME)

#define flick_corba_encode_mu_error(retname, errval, ENCNAME, LINKNAME)	\
		flick_error("CORBA", "encode", "marshal/unmarshal")

#define flick_corba_decode_mu_error(retname, errval, ENCNAME, LINKNAME)	\
		flick_error("CORBA", "decode", "marshal/unmarshal")

#define flick_corba_other_mu_error(retname, errval, ENCNAME, LINKNAME)	\
		flick_error("CORBA", "other", "marshal/unmarshal")

#endif /* __flick_pres_corba_h */

/* End of file. */

